import { CommonModule } from '@angular/common';
import { ɵɵdefineInjectable, Injectable, InjectionToken, Component, ViewEncapsulation, ChangeDetectionStrategy, ChangeDetectorRef, Optional, Inject, Input, ViewChild, ElementRef, Self, Attribute, ContentChildren, Pipe, EventEmitter, Directive, Output, NgModule } from '@angular/core';
import { NgForm, FormGroupDirective, NgControl, FormsModule } from '@angular/forms';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { Subject } from 'rxjs';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { mixinDisabled, mixinTabIndex, mixinErrorState, ErrorStateMatcher } from '@angular/material/core';
import { takeUntil } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { DomSanitizer } from '@angular/platform-browser';

var Direction;
(function (Direction) {
    Direction[Direction["default"] = 0] = "default";
    Direction[Direction["leftToRight"] = 1] = "leftToRight";
    Direction[Direction["rightToLeft"] = 2] = "rightToLeft";
})(Direction || (Direction = {}));

class BrowserService {
    constructor() {
        this.isEdge = /Edge\/\d./i.test(window.navigator.userAgent);
    }
}
BrowserService.ɵprov = ɵɵdefineInjectable({ factory: function BrowserService_Factory() { return new BrowserService(); }, token: BrowserService, providedIn: "root" });
BrowserService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
BrowserService.ctorParameters = () => [];

class DirectionService {
    // It is better to have this as a service rather than in every option instance
    isRTL(text) {
        const ltrChars = 'A-Za-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02B8\u0300-\u0590\u0800-\u1FFF' + '\u2C00-\uFB1C\uFDFE-\uFE6F\uFEFD-\uFFFF';
        const rtlChars = '\u0591-\u07FF\uFB1D-\uFDFD\uFE70-\uFEFC';
        const rtlDirCheck = new RegExp('^[^' + ltrChars + ']*[' + rtlChars + ']');
        return rtlDirCheck.test(text);
    }
}
DirectionService.ɵprov = ɵɵdefineInjectable({ factory: function DirectionService_Factory() { return new DirectionService(); }, token: DirectionService, providedIn: "root" });
DirectionService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

const SELECT_ELEMENT = new InjectionToken('OPTION_PARENT_COMPONENT');
let nextUniqueId = 0;
class SimpleOptionBase {
}
// eslint-disable-next-line @typescript-eslint/naming-convention
const _SimpleSelectMixinBase = mixinDisabled(SimpleOptionBase);
class OptionComponent extends _SimpleSelectMixinBase {
    constructor(cdRef, browser, dirService, parent) {
        super();
        this.cdRef = cdRef;
        this.browser = browser;
        this.dirService = dirService;
        this.parent = parent;
        this.highlight = new Subject();
        this.select = new Subject();
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.Direction = Direction;
        this.selected = false;
        this.hovered = false;
        this.overflowOffset = 0;
        this.scrollSideways = false;
        this._highlighted = false;
        this.uid = `simple-option-${++nextUniqueId}`;
        this.clientWidth = -1;
        // Force setter to be called in case id was not specified.
        this.id = this.id;
    }
    get id() { return this._id; }
    set id(value) {
        this._id = value || this.uid;
    }
    get text() {
        const contentChildNodes = this.content.nativeElement.childNodes;
        if (contentChildNodes.length > 0) {
            const optionNodes = contentChildNodes.childNodes;
            if (optionNodes) {
                let str = '';
                for (const optionNode of optionNodes) {
                    if (optionNode.nodeType === 3) {
                        str += optionNode.textContent;
                    }
                }
                return str;
            }
        }
        return this.content.nativeElement.textContent;
    }
    // Used in select.component.html
    get html() {
        if (this.content) {
            return this.content.nativeElement.innerHTML;
        }
        return '';
    }
    get isRTL() {
        return this.dirService.isRTL(this.text);
    }
    get direction() {
        if (!this.dir && this.parent && this.parent.dir) {
            this.dir = this.parent.dir;
        }
        if (this.dir) {
            switch (this.dir) {
                case 'rtl':
                    return Direction.rightToLeft;
                case 'auto':
                    return this.isRTL ? Direction.rightToLeft : Direction.leftToRight;
                default:
                    return Direction.leftToRight;
            }
        }
        return Direction.default;
    }
    get highlighted() {
        return this._highlighted;
    }
    set highlighted(highlighted) {
        this._highlighted = highlighted;
        this.cdRef.detectChanges();
    }
    shouldScrollSideways() {
        return this.highlighted && this.hovered && this.scrollSideways;
    }
    startFunctionality() {
        this.hovered = true;
        // Should be always calculated in case the window size changed
        this.calculateScroll();
        if (!this.highlighted) {
            this.highlight.next();
        }
    }
    stopFunctionality() {
        this.hovered = false;
        this.cdRef.markForCheck(); // Necessary to display sideway scroll properly
        this.cdRef.detectChanges(); // Necessary to display scroll sideways properly after blur
    }
    calculateScroll() {
        const element = this.element.nativeElement;
        // Only recalculate when the client width changed
        if (element.clientWidth !== this.clientWidth) {
            // Set clientWidth to the current width
            this.clientWidth = element.clientWidth;
            // Edge gonna be Edge...
            // Simple greater than comparison does not work in Edge
            // It seems to be a floating point issue
            // Microsoft Edge 44.18362.449.0
            if (!this.browser.isEdge && (element.scrollWidth > element.clientWidth) ||
                (element.scrollWidth - element.clientWidth) > 1) {
                this.scrollSideways = true;
                this.overflowOffset = element.clientWidth - element.scrollWidth;
                this.overflowOffset -= 4; // Just moving it 4px away from the scrollbar, for readability
                // Update scroll sideways transition
                this.cdRef.markForCheck();
                this.cdRef.detectChanges();
                if (this.direction === Direction.rightToLeft) {
                    this.overflowOffset = -this.overflowOffset;
                }
            }
            else {
                this.scrollSideways = false;
                this.overflowOffset = 0;
            }
        }
    }
}
OptionComponent.decorators = [
    { type: Component, args: [{
                selector: 'simple-option',
                exportAs: 'simpleOption',
                animations: [
                    trigger('scroll', [
                        state('initial', style({
                            transform: 'translateX(0)'
                        })),
                        state('scrolled', style({
                            transform: 'translateX({{offset}}px)'
                        }), { params: { offset: 0 } }),
                        transition('* => scrolled', [
                            animate('1500ms 250ms ease-in-out')
                        ])
                    ]),
                ],
                template: "<div class=\"simple-option-top\"></div>\r\n<div #optionField [ngClass]=\"{'simple-highlighted-option': highlighted}\" class=\"simple-option-container\">\r\n  <div [@scroll]=\"{value: shouldScrollSideways() ? 'scrolled' : 'initial', params: {offset: overflowOffset}}\">\r\n    <div #content class=\"simple-option-content\">\r\n      <ng-content></ng-content>\r\n    </div>\r\n  </div>\r\n</div>\r\n<div class=\"simple-option-bottom\"></div>\r\n",
                inputs: ['disabled'],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                // eslint-disable-next-line
                host: {
                    '[attr.id]': 'id',
                    class: 'simple-option',
                    role: 'option',
                    '[attr.aria-selected]': 'selected.toString()',
                    '[attr.aria-disabled]': 'disabled.toString()',
                    '[class.simple-disabled]': 'disabled',
                    '[class.simple-option-rtl]': 'direction === Direction.rightToLeft',
                    '[class.simple-option-ltr]': 'direction === Direction.leftToRight',
                    '(click)': 'select.next()',
                    '(mouseleave)': 'stopFunctionality()',
                    '(mousemove)': 'startFunctionality()'
                }
            },] }
];
OptionComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: BrowserService },
    { type: DirectionService },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [SELECT_ELEMENT,] }] }
];
OptionComponent.propDecorators = {
    label: [{ type: Input }],
    value: [{ type: Input }],
    dir: [{ type: Input }],
    id: [{ type: Input }],
    element: [{ type: ViewChild, args: ['optionField', { static: true },] }],
    content: [{ type: ViewChild, args: ['content', { static: true },] }]
};

class DeviceService {
    constructor() {
        this.mobileOrTablet = false;
        this.mobileOrTablet = this.isMobileOrTablet();
    }
    isMobileOrTablet() {
        let check = false;
        (a => {
            // eslint-disable-next-line max-len
            if (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino|android|ipad|playbook|silk/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4))) {
                check = true;
            }
            // eslint-disable-next-line @typescript-eslint/dot-notation
        })(navigator.userAgent || navigator.vendor || window['opera']);
        return check;
    }
}
DeviceService.ɵprov = ɵɵdefineInjectable({ factory: function DeviceService_Factory() { return new DeviceService(); }, token: DeviceService, providedIn: "root" });
DeviceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DeviceService.ctorParameters = () => [];

const INPUT_TIMEOUT = 500;
class ListKeyManager {
    constructor(list) {
        this.list = list;
        this.blur = new Subject();
        this.select = new Subject();
        this.timeoutHandler = null;
        this.searchString = '';
        this.numbersOnly = false;
        this.update();
    }
    catchKey(event) {
        const control = event.ctrlKey;
        const alt = event.altKey;
        const key = event.key;
        if (key) {
            switch (key) {
                case 'Backspace':
                    // Ignore
                    break;
                case 'Enter':
                    this.pressedEnter();
                    break;
                case 'ArrowUp':
                    this.pressedArrowUp(alt);
                    break;
                case 'ArrowDown':
                    this.pressedArrowDown(alt);
                    break;
                case 'Escape':
                    this.blur.next();
                    break;
                case 'PageUp':
                case 'Home':
                    this.pressedGoToFirst();
                    break;
                case 'PageDown':
                case 'End':
                    this.pressedGoToLast();
                    break;
                default:
                    this.pressedSomeButton(key, alt, control);
                    return;
            }
            event.preventDefault();
        }
    }
    pressedSomeButton(key, alt, control) {
        if (key.length === 1 && !alt && !control) {
            this.search(key);
        }
    }
    pressedGoToFirst() {
        for (let i = 0; i < this.list.elements.length; i++) {
            if (!this.list.skipPredicateFn(this.list.elements[i])) {
                this.select.next(i);
                return;
            }
        }
    }
    pressedGoToLast() {
        for (let i = this.list.elements.length - 1; i >= 0; i--) {
            if (!this.list.skipPredicateFn(this.list.elements[i])) {
                this.select.next(i);
                return;
            }
        }
    }
    pressedArrowUp(alt) {
        if (alt) {
            this.blur.next();
            return;
        }
        for (let i = this.list.highlightedIndex - 1; i >= 0; i--) {
            if (!this.list.skipPredicateFn(this.list.elements[i])) {
                this.select.next(i);
                return;
            }
        }
    }
    pressedArrowDown(alt) {
        if (alt) {
            this.blur.next();
            return;
        }
        for (let i = this.list.highlightedIndex + 1; i < this.list.elements.length; i++) {
            if (!this.list.skipPredicateFn(this.list.elements[i])) {
                this.select.next(i);
                return;
            }
        }
    }
    search(key) {
        if (this.list.elements.length > 0) {
            key = key.toUpperCase();
            let searchIndex;
            if (this.list.highlightedIndex < 0) {
                searchIndex = 0;
            }
            else {
                searchIndex = this.list.highlightedIndex;
            }
            // Change search behaviour for lists that only constist of numbers, to be able to search for e.g. 33 by pressing 2 x 3
            if (this.searchString.length > 0 && (this.searchString !== key || this.numbersOnly)) {
                this.searchString += key;
            }
            else {
                this.searchString = key;
                searchIndex++;
            }
            clearTimeout(this.timeoutHandler);
            this.timeoutHandler = null;
            this.timeoutHandler = setTimeout(() => {
                this.searchString = '';
            }, INPUT_TIMEOUT);
            for (let i = searchIndex; i < this.list.elements.length; i++) {
                if (!this.list.skipPredicateFn(this.list.elements[i]) && this.list.elements[i].text.toUpperCase().startsWith(this.searchString)) {
                    this.select.next(i);
                    return;
                }
            }
            for (let i = 0; i < searchIndex; i++) {
                // TODO: Change toUpperCase because not usable in every language
                if (!this.list.skipPredicateFn(this.list.elements[i]) && this.list.elements[i].text.toUpperCase().startsWith(this.searchString)) {
                    this.select.next(i);
                    return;
                }
            }
            // If not found, reset search
            this.searchString = '';
        }
    }
    // Enter always selects the current highlightedIndex
    pressedEnter() {
        this.select.next(this.list.highlightedIndex);
        this.blur.next();
    }
    hasNumbersOnly() {
        for (const option of this.list.elements) {
            if (Number.isNaN(Number(option.text))) {
                return false;
            }
        }
        return true;
    }
    update() {
        this.numbersOnly = this.hasNumbersOnly();
    }
}

class ListScrollManager {
    constructor(scrollableList) {
        this.scrollableList = scrollableList;
    }
    scrollDown(element, offset) {
        element.nativeElement.scrollTop += Math.round(offset);
    }
    scrollUp(element, offset) {
        element.nativeElement.scrollTop -= Math.round(offset);
    }
    offsetTop(element) {
        return element.nativeElement.getBoundingClientRect().top;
    }
    offsetBottom(element) {
        return element.nativeElement.getBoundingClientRect().bottom;
    }
    correctScroll(index) {
        if (index >= 0 &&
            this.scrollableList.elements.length > 0 &&
            this.scrollableList.list &&
            this.scrollableList.elements.length > index) {
            const selection = this.scrollableList.elements[index];
            const dropdownTop = this.offsetTop(this.scrollableList.list);
            const elementTop = this.offsetTop(selection.element);
            const dropdownBottom = this.offsetBottom(this.scrollableList.list);
            const elementBottom = this.offsetBottom(selection.element);
            if (elementBottom > dropdownBottom) {
                this.scrollDown(this.scrollableList.list, elementBottom - dropdownBottom);
            }
            else if (elementTop < dropdownTop) {
                this.scrollUp(this.scrollableList.list, dropdownTop - elementTop);
            }
        }
    }
    scrollClippedElement(index) {
        if (index >= 0 &&
            this.scrollableList.elements.length > 0 &&
            this.scrollableList.list &&
            this.scrollableList.elements.length > index) {
            const selection = this.scrollableList.elements[index];
            const dropdownTop = this.offsetTop(this.scrollableList.list);
            const elementTop = this.offsetTop(selection.element);
            const dropdownBottom = this.offsetBottom(this.scrollableList.list);
            const elementBottom = this.offsetBottom(selection.element);
            if (elementBottom > dropdownBottom && elementTop < dropdownBottom) {
                this.scrollDown(this.scrollableList.list, elementBottom - dropdownBottom);
            }
            else if (elementTop < dropdownTop && elementBottom > dropdownTop) {
                this.scrollUp(this.scrollableList.list, dropdownTop - elementTop);
            }
        }
    }
    scrollElementTop(index) {
        if (index >= 0 &&
            this.scrollableList.elements.length > 0 &&
            this.scrollableList.list &&
            this.scrollableList.elements.length > index) {
            const selection = this.scrollableList.elements[index];
            const dropdownTop = this.offsetTop(this.scrollableList.list);
            const elementTop = this.offsetTop(selection.element);
            this.scrollUp(this.scrollableList.list, dropdownTop - elementTop);
        }
    }
}

let nextUniqueId$1 = 0;
class SimpleSelectBase {
    constructor(_elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl) {
        this._elementRef = _elementRef;
        this._defaultErrorStateMatcher = _defaultErrorStateMatcher;
        this._parentForm = _parentForm;
        this._parentFormGroup = _parentFormGroup;
        this.ngControl = ngControl;
    }
}
// eslint-disable-next-line @typescript-eslint/naming-convention
const _SimpleSelectMixinBase$1 = mixinTabIndex(mixinDisabled(mixinErrorState(SimpleSelectBase)));
class SelectComponent extends _SimpleSelectMixinBase$1 {
    constructor(cdRef, browser, device, elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl, tabIndex) {
        super(elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl);
        this.cdRef = cdRef;
        this.browser = browser;
        this.device = device;
        this.ngControl = ngControl;
        // Enum for usage in template
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.Direction = Direction;
        this.focus = false;
        this.mouseOver = false;
        this.selectedIndex = -1;
        this.highlightedIndex = -1;
        this._hoverBorder = false;
        this._animate = true;
        this._required = false;
        this._elements = [];
        this.optionSubscriptions = [];
        this.uid = `simple-select-${++nextUniqueId$1}`;
        // Emits whenever the component is destroyed
        this.destroy = new Subject();
        this.skipPredicateFn = (item) => item.disabled;
        this.propagateChange = (_) => { };
        this.propagateTouched = () => { };
        if (this.ngControl) {
            // Note: we provide the value accessor through here, instead of
            // the `providers` to avoid running into a circular import.
            this.ngControl.valueAccessor = this;
        }
        this.scrollManager = new ListScrollManager(this);
        this.keyManager = new ListKeyManager(this);
        this.keyManager.blur.pipe(takeUntil(this.destroy)).subscribe(() => {
            this.blur();
        });
        this.keyManager.select.pipe(takeUntil(this.destroy)).subscribe((index) => {
            this.selectIndex(index);
            this.emit();
            this.scrollManager.correctScroll(this.highlightedIndex);
        });
        this.tabIndex = parseInt(tabIndex, 10) || 0;
        // Force setter to be called in case id was not specified.
        this.id = this.id;
    }
    get id() { return this._id; }
    set id(id) {
        this._id = id || this.uid;
    }
    get value() { return this._value; }
    set value(value) {
        this.writeValue(value);
    }
    get hoverBorder() { return this._hoverBorder; }
    set hoverBorder(hoverBorder) {
        this._hoverBorder = coerceBooleanProperty(hoverBorder);
    }
    get animate() { return this._animate; }
    set animate(animateVal) {
        this._animate = coerceBooleanProperty(animateVal);
    }
    get required() { return this._required; }
    set required(required) {
        this._required = coerceBooleanProperty(required);
    }
    get showUI() {
        return !this.hoverBorder || this.focus || this.mouseOver || this.disabled;
    }
    get showArrowAnimation() {
        return this.focus;
    }
    get showList() {
        return this.elements.length > 0 && this.focus;
    }
    get selected() {
        if (this.selectedIndex >= 0 && this.elements.length > this.selectedIndex) {
            return this.elements[this.selectedIndex];
        }
        return null;
    }
    get direction() {
        if (this.dir) {
            if (this.dir === 'auto') {
                if (this.elements) {
                    for (const option of this.elements) {
                        if (option.isRTL) {
                            return Direction.rightToLeft;
                        }
                    }
                    return Direction.leftToRight;
                }
            }
        }
        if (this.dir === 'rtl') {
            return Direction.rightToLeft;
        }
        else if (this.dir === 'ltr') {
            return Direction.leftToRight;
        }
        return Direction.default;
    }
    get ariaLabel() {
        return this._ariaLabel || this.placeholder;
    }
    get activeDescendant() {
        if (this.selectedIndex >= 0 && this.elements[this.selectedIndex]) {
            return this.elements[this.selectedIndex].id;
        }
        return null;
    }
    get elements() {
        return this._elements;
    }
    set elements(options) {
        this._elements = options;
    }
    startFunctionality() {
        // If not disabled and not already focused
        if (!this.disabled && !this.focus) {
            this.highlightIndex(this.selectedIndex);
            this.focus = true;
        }
    }
    stopFunctionality() {
        // Call stopFunctionality of the highlighted option to stop sideway scroll
        if (this.highlightedIndex >= 0 && this.highlightedIndex < this.elements.length) {
            this.elements[this.highlightedIndex].stopFunctionality();
        }
        this.focus = false;
        this.propagateTouched();
    }
    mouseenter() {
        this.mouseOver = true;
    }
    mouseleave() {
        this.mouseOver = false;
    }
    scrolledOutsideSelect() {
        if (!this.mouseOver && !this.disabled && this.focus) {
            this.blur();
        }
    }
    blur() {
        if (!this.browser.isEdge) {
            this._elementRef.nativeElement.blur();
        }
        else {
            // Edge workaround to maintain tabindex
            // After bluring an element in Edge, the tabindex resets
            // It works, but the focus stays and startFunctionality triggers after reselecting the browser window
            // Won't fix this - Edge should fix its bugs...
            this.stopFunctionality();
        }
    }
    renderedList() {
        this.scrollManager.scrollElementTop(this.highlightedIndex);
    }
    selectIndex(index) {
        if (index < this.elements.length) {
            if (index >= 0) {
                if (!this.skipPredicateFn(this.elements[index])) {
                    // Do not reselect, but highlight
                    if (this.selectedIndex !== index) {
                        if (this.selectedIndex >= 0) {
                            this.elements[this.selectedIndex].selected = false;
                        }
                        this.elements[index].selected = true;
                        this.selectedIndex = index;
                        this.emit();
                    }
                    this.highlightIndex(index);
                }
            }
            else {
                this.selectedIndex = index;
                this.highlightIndex(0);
                this.emit();
            }
        }
    }
    highlightIndex(index) {
        if (this.elements.length > 0 &&
            index >= 0 &&
            this.highlightedIndex !== index &&
            !this.skipPredicateFn(this.elements[index])) {
            if (this.highlightedIndex >= 0 && this.highlightedIndex < this.elements.length) {
                this.elements[this.highlightedIndex].highlighted = false;
            }
            this.elements[index].highlighted = true;
            this.highlightedIndex = index;
        }
    }
    initOptions() {
        if (this.options) {
            // Removing previous subscriptions
            for (const subscription of this.optionSubscriptions) {
                subscription.unsubscribe();
            }
            this.optionSubscriptions = [];
            // Initialize the elements
            this.elements = this.options.toArray();
            for (let i = 0; i < this.elements.length; i++) {
                const element = this.elements[i];
                this.optionSubscriptions.push(element.select.pipe(takeUntil(this.destroy)).subscribe(() => {
                    this.selectIndex(i);
                }));
                this.optionSubscriptions.push(element.highlight.pipe(takeUntil(this.destroy)).subscribe(() => {
                    this.highlightIndex(i);
                    this.scrollManager.scrollClippedElement(i);
                }));
            }
        }
    }
    emit() {
        if (this.selectedIndex >= 0 && this.selectedIndex < this.elements.length) {
            this._value = this.elements[this.selectedIndex].value;
        }
        else {
            this._value = null;
        }
        this.propagateChange(this._value);
    }
    writeValue(value) {
        if (value !== null) {
            this._value = value;
            if (this._elementRef) {
                for (let i = 0; i < this.elements.length; i++) {
                    if (this.elements[i].value === value) {
                        this.selectIndex(i);
                        return;
                    }
                }
            }
        }
        else {
            this.selectIndex(-1);
        }
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(fn) {
        this.propagateTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    ngAfterViewInit() {
        this.initOptions();
        // Reset highlightedIndex to 0 or to the selectedIndex
        this.highlightIndex((this.selectedIndex >= 0) ? this.selectedIndex : 0);
        this.options.changes.pipe(takeUntil(this.destroy)).subscribe(() => {
            this.updateOptions();
        });
        this.cdRef.detectChanges();
    }
    updateOptions() {
        // Reset the selection when the options changed
        this.initOptions();
        let selectedIndex = -1;
        // Select option based on previously selected value
        for (let i = 0; i < this.elements.length; i++) {
            if (this.elements[i].value === this._value) {
                selectedIndex = i;
                break;
            }
        }
        this.selectIndex(selectedIndex);
        this.keyManager.update();
        this.cdRef.markForCheck();
        this.ngDoCheck();
    }
    ngDoCheck() {
        if (this.ngControl) {
            this.updateErrorState();
        }
    }
    ngOnDestroy() {
        this.destroy.next();
        this.destroy.complete();
        this.stateChanges.complete();
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'simple-select',
                exportAs: 'simpleSelect',
                animations: [
                    trigger('spin', [
                        state('initial', style({
                            transform: 'rotate(0)'
                        })),
                        state('rotate', style({
                            transform: 'rotate({{degrees}}deg)'
                        }), { params: { degrees: -180 } }),
                        transition('initial <=> rotate', [
                            animate('.2s linear')
                        ])
                    ]),
                ],
                template: "<div [ngClass]=\"{'simple-select-rtl': direction === Direction.rightToLeft, 'simple-select-ltr': direction === Direction.leftToRight}\" class=\"simple-select-wrapper\">\r\n  <div [ngClass]=\"{'simple-select-border': showUI, 'simple-invalid': errorState}\" class=\"simple-select-bar\" (click)=\"startFunctionality()\" (mouseenter)=\"mouseenter()\" (mouseleave)=\"mouseleave()\">\r\n    <div class=\"simple-select-box\">\r\n      <div *ngIf=\"!selected; else showSelected\" class=\"simple-placeholder-text\">{{placeholder}}</div>\r\n      <ng-template #showSelected>\r\n        <div [innerHTML]=\"selected.html | noSanitize\" class=\"simple-select-selection\"></div>\r\n      </ng-template>\r\n    </div>\r\n    <div [hidden]=\"!showUI\" [@spin]=\"{value: animate && showArrowAnimation ? 'rotate' : 'initial', params: {degrees: direction === Direction.rightToLeft ? 180 : -180}}\" class=\"simple-arrow-down\"></div>\r\n  </div>\r\n  <div class=\"simple-options-container\">\r\n    <div [ngClass]=\"{'simple-select-inactive': !showList}\" class=\"simple-options-wrapper\" (click)=\"blur()\">\r\n      <div #optionList class=\"simple-option-list\">\r\n        <ng-container simpleRendered *ngIf=\"showList\" (renderComplete)=\"renderedList()\">\r\n          <ng-content></ng-content>\r\n        </ng-container>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n",
                inputs: ['disabled', 'tabIndex'],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    {
                        provide: SELECT_ELEMENT,
                        useExisting: SelectComponent
                    }
                ],
                // eslint-disable-next-line
                host: {
                    '[attr.id]': 'id',
                    '[attr.tabindex]': 'tabIndex',
                    class: 'simple-select',
                    role: 'listbox',
                    '[attr.aria-label]': 'ariaLabel',
                    '[attr.aria-required]': 'required.toString()',
                    '[attr.aria-disabled]': 'disabled.toString()',
                    '[attr.aria-invalid]': 'errorState',
                    '[attr.aria-activedescendant]': 'activeDescendant',
                    '[class.simple-disabled]': 'disabled',
                    '[class.simple-required]': 'required',
                    '[class.simple-select-mobile-tablet]': 'device.mobileOrTablet',
                    '(blur)': 'stopFunctionality()',
                    '(document:scroll)': 'scrolledOutsideSelect()',
                    '(focus)': 'startFunctionality()',
                    '(keydown)': 'keyManager.catchKey($event)'
                }
            },] }
];
SelectComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: BrowserService },
    { type: DeviceService },
    { type: ElementRef },
    { type: ErrorStateMatcher },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
];
SelectComponent.propDecorators = {
    placeholder: [{ type: Input }],
    dir: [{ type: Input }],
    errorStateMatcher: [{ type: Input }],
    _ariaLabel: [{ type: Input, args: ['aria-label',] }],
    id: [{ type: Input }],
    value: [{ type: Input }],
    hoverBorder: [{ type: Input }],
    animate: [{ type: Input }],
    required: [{ type: Input }],
    list: [{ type: ViewChild, args: ['optionList', { static: true },] }],
    options: [{ type: ContentChildren, args: [OptionComponent, { descendants: true },] }]
};

class NoSanitizePipe {
    constructor(domSanitizer) {
        this.domSanitizer = domSanitizer;
    }
    transform(html) {
        return this.domSanitizer.bypassSecurityTrustHtml(html);
    }
}
NoSanitizePipe.decorators = [
    { type: Pipe, args: [{ name: 'noSanitize' },] }
];
NoSanitizePipe.ctorParameters = () => [
    { type: DomSanitizer }
];

class RenderedDirective {
    constructor() {
        this.renderComplete = new EventEmitter();
    }
    ngAfterViewInit() {
        this.renderComplete.emit();
    }
}
RenderedDirective.decorators = [
    { type: Directive, args: [{
                selector: '[simpleRendered]'
            },] }
];
RenderedDirective.propDecorators = {
    renderComplete: [{ type: Output }]
};

class SimpleSelectModule {
}
SimpleSelectModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SelectComponent, OptionComponent, NoSanitizePipe, RenderedDirective],
                imports: [
                    CommonModule,
                    BrowserAnimationsModule
                ],
                providers: [
                    DeviceService
                ],
                exports: [SelectComponent, OptionComponent, FormsModule]
            },] }
];

/*
 * Public API Surface of simple-select
 */

/**
 * Generated bundle index. Do not edit.
 */

export { OptionComponent, SelectComponent, SimpleSelectModule, SELECT_ELEMENT as ɵa, BrowserService as ɵb, DirectionService as ɵc, DeviceService as ɵe, NoSanitizePipe as ɵf, RenderedDirective as ɵg };
//# sourceMappingURL=nutrify-ngx-simple-select.js.map
